// acl.cpp:
//
// AccessControlList class implementation
//
// jim frost 04.06.95

#include "security.hpp"

AccessControlList::AccessControlList(void)
{
  acl = NULL;
}

AccessControlList::AccessControlList(const ACL* _acl)
{
  if (_acl == NULL) {
    acl = NULL;
    return;
  }
  else {
    // copy the ACL.
    acl = (ACL*)new char[_acl->AclSize];
    memcpy(acl, _acl, _acl->AclSize);
  }
}

AccessControlList::AccessControlList(const AccessControlList* _acl)
{
  if ((_acl == NULL) || (_acl->acl == NULL)) {
    acl = NULL;
    return;
  }
  else {
    acl = (ACL*)new char[_acl->acl->AclSize];
    memcpy(acl, _acl->acl, _acl->acl->AclSize);
  }
}

AccessControlList::~AccessControlList(void)
{
  if (acl != NULL)
    delete[] (char*)acl;
}

unsigned int
AccessControlList::GetLength(void)
{
  return acl->AceCount;
}

void
AccessControlList::AddAce(AceUnion* new_ace)
{
  // if we don't have an acl already, create one
  if (acl == NULL) {
    acl = (ACL*)new char[sizeof(ACL) + new_ace->header.AceSize];
    acl->AclRevision = ACL_REVISION;
    acl->Sbz1 = 0;
    acl->AclSize = sizeof(ACL) + new_ace->header.AceSize;
    acl->AceCount = 0;
    acl->Sbz2 = 0;
  }

  if (!::AddAce(acl, ACL_REVISION, 0, new_ace, new_ace->header.AceSize) &&
      (GetLastError() == ERROR_INSUFFICIENT_BUFFER)) {

    // acl isn't big enough to accommodate the addition -- enlarge it

    unsigned int new_size = acl->AclSize + new_ace->header.AceSize;

    ACL* new_acl = (ACL*)new char[new_size];
    memset(new_acl, 0, new_size);
    memcpy(new_acl, acl, acl->AclSize);
    new_acl->AclSize = new_size;
    delete[] acl;
    acl = new_acl;

    ::AddAce(acl, ACL_REVISION, 0, new_ace, new_ace->header.AceSize);
  }
}

void
AccessControlList::AppendAce(AceUnion* new_ace)
{
  // if we don't have an acl already, create one
  if (acl == NULL) {
    acl = (ACL*)new char[sizeof(ACL) + new_ace->header.AceSize];
    acl->AclRevision = ACL_REVISION;
    acl->Sbz1 = 0;
    acl->AclSize = sizeof(ACL) + new_ace->header.AceSize;
    acl->AceCount = 0;
    acl->Sbz2 = 0;
  }

  if (!::AddAce(acl, ACL_REVISION, MAXDWORD, new_ace, new_ace->header.AceSize) &&
      (GetLastError() == ERROR_INSUFFICIENT_BUFFER)) {

    // acl isn't big enough to accommodate the addition -- enlarge it

    unsigned int new_size = acl->AclSize + new_ace->header.AceSize;

    ACL* new_acl = (ACL*)new char[new_size];
    memset(new_acl, 0, acl->AclSize);
    memcpy(new_acl, acl, acl->AclSize);
    new_acl->AclSize = new_size;
    delete[] acl;
    acl = new_acl;

    ::AddAce(acl, ACL_REVISION, MAXDWORD, new_ace, new_ace->header.AceSize);
  }
}

void
AccessControlList::AddAllowAce(unsigned char flags,
			       SecurityIdentifier* sid,
			       DWORD access_mask)
{
  int ace_size = sizeof(ACCESS_ALLOWED_ACE) - sizeof(DWORD) + GetLengthSid(sid->sid);

  AceUnion* ace = (AceUnion*)new char[ace_size];
  ace->header.AceType = ACCESS_ALLOWED_ACE_TYPE;
  ace->header.AceFlags = flags;
  ace->header.AceSize = ace_size;
  ace->allow.Mask = access_mask;
  CopySid(GetLengthSid(sid->sid), &ace->allow.SidStart, sid->sid);
  AddAce(ace);
}

void
AccessControlList::AddDenyAce(unsigned char flags,
			      SecurityIdentifier* sid,
			      DWORD access_mask)
{
  int ace_size = sizeof(ACCESS_DENIED_ACE) - sizeof(DWORD) + GetLengthSid(sid->sid);

  AceUnion* ace = (AceUnion*)new char[ace_size];
  ace->header.AceType = ACCESS_DENIED_ACE_TYPE;
  ace->header.AceFlags = flags;
  ace->header.AceSize = ace_size;
  ace->deny.Mask = access_mask;
  CopySid(GetLengthSid(sid->sid), &ace->deny.SidStart, sid->sid);
  AddAce(ace);
}

void
AccessControlList::AppendAllowAce(unsigned char flags,
				  SecurityIdentifier* sid,
				  DWORD access_mask)
{
  int ace_size = sizeof(ACCESS_ALLOWED_ACE) - sizeof(DWORD) + GetLengthSid(sid->sid);

  AceUnion* ace = (AceUnion*)new char[ace_size];
  ace->header.AceType = ACCESS_ALLOWED_ACE_TYPE;
  ace->header.AceFlags = flags;
  ace->header.AceSize = ace_size;
  ace->allow.Mask = access_mask;
  CopySid(GetLengthSid(sid->sid), &ace->allow.SidStart, sid->sid);
  AppendAce(ace);
}

void
AccessControlList::AppendDenyAce(unsigned char flags,
				 SecurityIdentifier* sid,
				 DWORD access_mask)
{
  int ace_size = sizeof(ACCESS_DENIED_ACE) - sizeof(DWORD) + GetLengthSid(sid->sid);

  AceUnion* ace = (AceUnion*)new char[ace_size];
  ace->header.AceType = ACCESS_DENIED_ACE_TYPE;
  ace->header.AceFlags = flags;
  ace->header.AceSize = ace_size;
  ace->deny.Mask = access_mask;
  CopySid(GetLengthSid(sid->sid), &ace->deny.SidStart, sid->sid);
  AppendAce(ace);
}

void
AccessControlList::DeleteAce(unsigned int index)
{
  ::DeleteAce(acl, index);
}

unsigned int
AccessControlList::GetAce(unsigned int index, AccessControlEntry& ret_ace)
{
  AceUnion* ace;

  if (!::GetAce(acl, index, (void**)&ace))
    return 0;

  ret_ace.SetType(ace->header.AceType);
  ret_ace.SetFlags(ace->header.AceFlags);

  switch (ace->header.AceType) {
  case ACCESS_ALLOWED_ACE_TYPE:
    ret_ace.SetMask(ace->allow.Mask);
    ret_ace.SetSecurityIdentifier(new SecurityIdentifier((PSID)&ace->allow.SidStart));
    break;
  case ACCESS_DENIED_ACE_TYPE:
    ret_ace.SetMask(ace->deny.Mask);
    ret_ace.SetSecurityIdentifier(new SecurityIdentifier((PSID)&ace->deny.SidStart));
    break;
  default:
    // unsupported type
    return 1;
  }
  return 1;
}
