// sid.cpp:
//
// security identifier class implementation
//
// jim frost 04.06.95

#include "security.hpp"

SecurityIdentifier* CreateSecurityIdentifier(const char* account)
{
  char sid_buffer[1024];
  DWORD sid_size = sizeof(sid_buffer);
  PSID sid = (PSID)sid_buffer;
  char domain_buffer[1024];
  DWORD domain_size = sizeof(domain_buffer);
  char* domain = domain_buffer;
  SID_NAME_USE snu; // dummy

  while (!LookupAccountName(NULL, account,
			    sid, &sid_size,
			    domain, &domain_size,
			    &snu)) {
    if (sid != sid_buffer)
      delete[] (char*)sid;
    if (domain != domain_buffer)
      delete[] domain;

    switch (GetLastError()) {
    case ERROR_INSUFFICIENT_BUFFER:
      sid = (PSID)new char[sid_size];
      domain = new char[domain_size];
      break;
    default:
      return NULL;
    }
  }

  SecurityIdentifier* new_sid = new SecurityIdentifier(sid);

  if (sid != sid_buffer)
    delete[] (char*)sid;

  return new_sid;
}

SecurityIdentifier::SecurityIdentifier(PSID _sid)
{
  sid = new char[GetLengthSid(_sid)];
  CopySid(GetLengthSid(_sid), sid, _sid);
  account = NULL;
  domain = NULL;
}

SecurityIdentifier::SecurityIdentifier(const SecurityIdentifier* new_sid)
{
  if (new_sid->sid == NULL)
    sid = NULL;
  else {
    sid = (PSID)new char[GetLengthSid(new_sid->sid)];
    CopySid(GetLengthSid(new_sid->sid), sid, new_sid->sid);
  }
  account = NULL;
  domain = NULL;
}

SecurityIdentifier::~SecurityIdentifier(void)
{
  delete[] sid;
  if (account != NULL)
    delete[] account;
  if (domain != NULL)
    delete[] domain;
}

const char*
SecurityIdentifier::GetAccount(void)
{
  if (account != NULL)
    return account;

  char         account_buffer[1024];
  DWORD account_size = sizeof(account_buffer);
  char         domain_buffer[1024];
  DWORD domain_size = sizeof(domain_buffer);
  SID_NAME_USE snu; // dummy

  account = account_buffer;
  domain = domain_buffer;

  while (!LookupAccountSid(NULL, sid,
			   account, &account_size,
			   domain, &domain_size,
			   &snu)) {
    if (account != account_buffer)
      delete[] account;
    if (domain != domain_buffer)
      delete[] domain;
    switch (GetLastError()) {
    case ERROR_INSUFFICIENT_BUFFER:
      account = new char[account_size];
      domain = new char[domain_size];
      break;
    default:
      account = new char[10];
      strcpy(account, "<unknown>");
      domain = new char[10];
      strcpy(account, "<unknown>");
      return account;
    }
  }

  // if the static buffer was used, copy the information into a new buffer
  if (account == account_buffer) {
    account = new char[strlen(account_buffer) + 1];
    strcpy(account, account_buffer);
  }
  if (domain == domain_buffer) {
    domain = new char[strlen(domain_buffer) + 1];
    strcpy(domain, domain_buffer);
  }

  return account;
}

const char*
SecurityIdentifier::GetDomain(void)
{
  if (domain == NULL)
    (void)GetAccount(); // go get account and domain info
  return domain;
}
